@@ -1,4 +1,4 @@
-from .core.alignment import align, concat
+from .core.alignment import align, broadcast_arrays, concat
 from .core.variable import Variable, Coordinate
 from .core.dataset import Dataset, open_dataset
 from .core.dataarray import DataArray
