@@ -1,3 +1,4 @@
+import functools
 import numpy as np
 import pandas as pd
 import warnings
@@ -8,6 +9,7 @@ from .core import indexing, utils
 from .core.variable import as_variable, Variable
 from .core.pycompat import iteritems, bytes_type, unicode_type, OrderedDict
 
+
 # standard calendars recognized by netcdftime
 _STANDARD_CALENDARS = set(['standard', 'gregorian', 'proleptic_gregorian'])
 
@@ -153,9 +155,9 @@ def guess_time_units(dates):
     """
     dates = pd.DatetimeIndex(np.asarray(dates).reshape(-1))
     unique_timedeltas = np.unique(np.diff(dates.values))
-    for time_unit, delta in [('days', '1 days'), ('hours', '3600s'),
-                             ('minutes', '60s'), ('seconds', '1s')]:
-        unit_delta = pd.to_timedelta(delta)
+    for time_unit, delta in [('days', 86400), ('hours', 3600),
+                             ('minutes', 60), ('seconds', 1)]:
+        unit_delta = np.timedelta64(10 ** 9 * delta, 'ns')
         diffs = unique_timedeltas / unit_delta
         if np.all(diffs == diffs.astype(int)):
             break
@@ -187,17 +189,19 @@ def encode_cf_datetime(dates, units=None, calendar=None):
     netCDF4.date2num
     """
     import netCDF4 as nc4
+
+    dates = np.asarray(dates)
+
     if units is None:
         units = guess_time_units(dates)
     if calendar is None:
         calendar = 'proleptic_gregorian'
 
-    if (isinstance(dates, np.ndarray)
-            and np.issubdtype(dates.dtype, np.datetime64)):
+    if np.issubdtype(dates.dtype, np.datetime64):
         # for now, don't bother doing any trickery like decode_cf_datetime to
         # convert dates to numbers faster
         # note: numpy's broken datetime conversion only works for us precision
-        dates = np.asarray(dates).astype('M8[us]').astype(datetime)
+        dates = dates.astype('M8[us]').astype(datetime)
 
     if hasattr(dates, 'ndim') and dates.ndim == 0:
         # unpack dates because date2num doesn't like 0-dimensional arguments
@@ -371,6 +375,73 @@ def pop_to(source, dest, key, default=None):
     return value
 
 
+def _var_as_tuple(var):
+    return var.dims, var.values, var.attrs.copy(), var.encoding.copy()
+
+
+def maybe_encode_datetime(var):
+    if (np.issubdtype(var.dtype, np.datetime64)
+            or (var.dtype.kind == 'O'
+                and isinstance(var.values.flat[0], datetime))):
+
+        dims, values, attrs, encoding = _var_as_tuple(var)
+        if 'units' in attrs or 'calendar' in attrs:
+            raise ValueError(
+                "Failed hard to prevent overwriting 'units' or 'calendar'")
+
+        (values, units, calendar) = encode_cf_datetime(
+            values, encoding.pop('units', None), encoding.pop('calendar', None))
+        attrs['units'] = units
+        attrs['calendar'] = calendar
+        var = Variable(dims, values, attrs, encoding)
+    return var
+
+
+def maybe_encode_offset_and_scale(var, needs_copy=True):
+    if any(k in var.encoding for k in ['add_offset', 'scale_factor']):
+        dims, values, attrs, encoding = _var_as_tuple(var)
+        values = np.array(values, dtype=float, copy=needs_copy)
+        needs_copy = False
+        if 'add_offset' in encoding:
+            values -= pop_to(encoding, attrs, 'add_offset')
+        if 'scale_factor' in encoding:
+            values /= pop_to(encoding, attrs, 'scale_factor')
+        var = Variable(dims, values, attrs, encoding)
+    return var, needs_copy
+
+
+def maybe_encode_fill_value(var, needs_copy=True):
+    # replace NaN with the fill value
+    if '_FillValue' in var.encoding:
+        dims, values, attrs, encoding = _var_as_tuple(var)
+        fill_value = pop_to(encoding, attrs, '_FillValue')
+        if not pd.isnull(fill_value):
+            missing = pd.isnull(values)
+            if missing.any():
+                if needs_copy:
+                    values = values.copy()
+                    needs_copy = False
+                values[missing] = fill_value
+        var = Variable(dims, values, attrs, encoding)
+    return var, needs_copy
+
+
+def maybe_encode_dtype(var, needs_copy=True):
+    if 'dtype' in var.encoding:
+        dims, values, attrs, encoding = _var_as_tuple(var)
+        dtype = np.dtype(encoding.pop('dtype'))
+        if dtype.kind != 'O':
+            if np.issubdtype(dtype, int):
+                out = np.empty_like(values) if needs_copy else values
+                np.around(values, out=out)
+            if dtype == 'S1' and values.dtype != 'S1':
+                values = string_to_char(np.asarray(values, 'S'))
+                dims = dims + ('string%s' % values.shape[-1],)
+            values = np.asarray(values, dtype=dtype)
+            var = Variable(dims, values, attrs, encoding)
+    return var
+
+
 def _infer_dtype(array):
     """Given an object array with no missing values, infer its dtype from its
     first element
@@ -389,85 +460,91 @@ def _infer_dtype(array):
     return dtype
 
 
-def encode_cf_variable(var):
-    """Converts an Variable into an Variable suitable for saving as a netCDF
-    variable
-    """
-    dimensions = var.dims
-    data = var.values
-    attributes = var.attrs.copy()
-    encoding = var.encoding.copy()
-
-    # convert datetimes into numbers
-    if (np.issubdtype(data.dtype, np.datetime64)
-            or (data.dtype.kind == 'O'
-                and isinstance(data.reshape(-1)[0], datetime))):
-        if 'units' in attributes or 'calendar' in attributes:
-            raise ValueError(
-                "Failed hard to prevent overwriting 'units' or 'calendar'")
-        (data, units, calendar) = encode_cf_datetime(
-            data, encoding.pop('units', None), encoding.pop('calendar', None))
-        attributes['units'] = units
-        attributes['calendar'] = calendar
-
-    # unscale/mask
-    if any(k in encoding for k in ['add_offset', 'scale_factor']):
-        data = np.array(data, dtype=float, copy=True)
-        if 'add_offset' in encoding:
-            data -= pop_to(encoding, attributes, 'add_offset')
-        if 'scale_factor' in encoding:
-            data /= pop_to(encoding, attributes, 'scale_factor')
-
-    # replace NaN with the fill value
-    if '_FillValue' in encoding:
-        fill_value = pop_to(encoding, attributes, '_FillValue')
-        if not pd.isnull(fill_value):
-            missing = pd.isnull(data)
-            if missing.any():
-                data = data.copy()
-                data[missing] = fill_value
-
-    # cast to encoded dtype
-    if 'dtype' in encoding:
-        dtype = np.dtype(encoding.pop('dtype'))
-        if dtype.kind != 'O':
-            if np.issubdtype(dtype, int):
-                data = data.round()
-            if dtype == 'S1' and data.dtype != 'S1':
-                data = string_to_char(np.asarray(data, 'S'))
-                dimensions = dimensions + ('string%s' % data.shape[-1],)
-            data = np.asarray(data, dtype=dtype)
-
-    # infer a valid dtype if necessary
-    # TODO: move this from conventions to backends (it's not CF related)
-    if data.dtype.kind == 'O':
-        missing = pd.isnull(data)
+def ensure_dtype_not_object(var):
+    # TODO: move this from conventions to backends? (it's not CF related)
+    if var.dtype.kind == 'O':
+        dims, values, attrs, encoding = _var_as_tuple(var)
+        missing = pd.isnull(values)
         if missing.any():
-            non_missing_data = data[~missing]
-            inferred_dtype = _infer_dtype(non_missing_data)
+            non_missing_values = values[~missing]
+            inferred_dtype = _infer_dtype(non_missing_values)
 
             if inferred_dtype.kind in ['S', 'U']:
                 # There is no safe bit-pattern for NA in typical binary string
-                # formats, we so can't set a _FillValue. Unfortunately, this
+                # formats, we so can't set a fill_value. Unfortunately, this
                 # means we won't be able to restore string arrays with missing
                 # values.
                 fill_value = ''
             else:
-                # insist on using float for numeric data
+                # insist on using float for numeric values
                 if not np.issubdtype(inferred_dtype, float):
                     inferred_dtype = np.dtype(float)
                 fill_value = np.nan
 
-            data = np.array(data, dtype=inferred_dtype, copy=True)
-            data[missing] = fill_value
+            values = np.array(values, dtype=inferred_dtype, copy=True)
+            values[missing] = fill_value
         else:
-            data = np.asarray(data, dtype=_infer_dtype(data))
+            values = np.asarray(values, dtype=_infer_dtype(values))
+        var = Variable(dims, values, attrs, encoding)
+    return var
 
-    return Variable(dimensions, data, attributes, encoding=encoding)
+
+def encode_cf_variable(var, needs_copy=True):
+    """
+    Converts an Variable into an Variable which follows some
+    of the CF conventions:
+
+        - Nans are masked using _FillValue (or the deprecated missing_value)
+        - Rescaling via: scale_factor and add_offset
+        - datetimes are converted to the CF 'units since time' format
+        - dtype encodings are enforced.
+
+    Parameters
+    ----------
+    var : xray.Variable
+        A variable holding un-encoded data.
+
+    Returns
+    -------
+    out : xray.Variable
+        A variable which has been encoded as described above.
+    """
+    var = maybe_encode_datetime(var)
+    var, needs_copy = maybe_encode_offset_and_scale(var, needs_copy)
+    var, needs_copy = maybe_encode_fill_value(var, needs_copy)
+    var = maybe_encode_dtype(var, needs_copy)
+    var = ensure_dtype_not_object(var)
+    return var
 
 
 def decode_cf_variable(var, concat_characters=True, mask_and_scale=True,
                        decode_times=True):
+    """
+    Decodes a variable which may hold CF encoded information.
+
+    This includes variables that have been masked and scaled, which
+    hold CF style time variables (this is almost always the case if
+    the dataset has been serialized) and which have strings encoded
+    as character arrays.
+
+    Parameters
+    ----------
+    var : Variable
+        A variable holding potentially CF encoded information.
+    concat_characters : bool
+        Should character arrays be concatenated to strings, for
+        example: ['h', 'e', 'l', 'l', 'o'] -> 'hello'
+    mask_and_scale: bool
+        Lazily scale (using scale_factor and add_offset) and mask
+        (using _FillValue).
+    decode_times : bool
+        Decode cf times ('hours since 2000-01-01') to np.datetime64.
+
+    Returns
+    -------
+    out : Variable
+        A variable holding the decoded equivalent of var
+    """
     # use _data instead of data so as not to trigger loading data
     var = as_variable(var)
     data = var._data
@@ -477,8 +554,9 @@ def decode_cf_variable(var, concat_characters=True, mask_and_scale=True,
 
     if 'dtype' in encoding:
         if data.dtype != encoding['dtype']:
-            raise ValueError("Refused to overwrite dtype")
-    encoding['dtype'] = data.dtype
+            warnings.warn("CF decoding is overwriting dtype")
+    else:
+        encoding['dtype'] = data.dtype
 
     if concat_characters:
         if data.dtype.kind == 'S' and data.dtype.itemsize == 1:
@@ -486,6 +564,14 @@ def decode_cf_variable(var, concat_characters=True, mask_and_scale=True,
             data = CharToStringArray(data)
 
     if mask_and_scale:
+        if 'missing_value' in attributes:
+            # missing_value is deprecated, but we still want to support it as
+            # an alias for _FillValue.
+            assert ('_FillValue' not in attributes
+                    or utils.equivalent(attributes['_FillValue'],
+                                        attributes['missing_value']))
+            attributes['_FillValue'] = attributes.pop('missing_value')
+
         fill_value = pop_to(attributes, encoding, '_FillValue')
         scale_factor = pop_to(attributes, encoding, 'scale_factor')
         add_offset = pop_to(attributes, encoding, 'add_offset')
@@ -510,7 +596,10 @@ def decode_cf_variable(var, concat_characters=True, mask_and_scale=True,
 
 def decode_cf_variables(variables, concat_characters=True, mask_and_scale=True,
                         decode_times=True):
-    """Decode a bunch of CF variables together.
+    """
+    Decode a several CF encoded variables.
+
+    See: decode_cf_variable
     """
     dimensions_used_by = defaultdict(list)
     for v in variables.values():
@@ -534,3 +623,73 @@ def decode_cf_variables(variables, concat_characters=True, mask_and_scale=True,
             v, concat_characters=concat, mask_and_scale=mask_and_scale,
             decode_times=decode_times)
     return new_vars
+
+
+def cf_decoder(variables, attributes,
+               concat_characters=True, mask_and_scale=True,
+               decode_times=True):
+    """
+    Decode a set of CF encoded variables and attributes.
+
+    See Also, decode_cf_variable
+
+    Parameters
+    ----------
+    variables : dict
+        A dictionary mapping from variable name to xray.Variable
+    attributes : dict
+        A dictionary mapping from attribute name to value
+    concat_characters : bool
+        Should character arrays be concatenated to strings, for
+        example: ['h', 'e', 'l', 'l', 'o'] -> 'hello'
+    mask_and_scale: bool
+        Lazily scale (using scale_factor and add_offset) and mask
+        (using _FillValue).
+    decode_times : bool
+        Decode cf times ('hours since 2000-01-01') to np.datetime64.
+    decode_cf : bool
+        If false this skips decoding.  This is around for backward
+        compatibility.
+
+    Returns
+    -------
+    decoded_variables : dict
+        A dictionary mapping from variable name to xray.Variable,
+    decoded_attributes : dict
+        A dictionary mapping from attribute name to value
+    """
+    new_vars = decode_cf_variables(variables, concat_characters,
+                                   mask_and_scale, decode_times)
+    return new_vars, attributes
+
+
+def cf_encoder(variables, attributes):
+    """
+    A function which takes a dicts of variables and attributes
+    and encodes them to conform to CF conventions as much
+    as possible.  This includes masking, scaling, character
+    array handling, and CF-time encoding.
+
+    Decode a set of CF encoded variables and attributes.
+
+    See Also, decode_cf_variable
+
+    Parameters
+    ----------
+    variables : dict
+        A dictionary mapping from variable name to xray.Variable
+    attributes : dict
+        A dictionary mapping from attribute name to value
+
+    Returns
+    -------
+    encoded_variables : dict
+        A dictionary mapping from variable name to xray.Variable,
+    encoded_attributes : dict
+        A dictionary mapping from attribute name to value
+
+    See also: encode_cf_variable
+    """
+    new_vars = OrderedDict((k, encode_cf_variable(v))
+                           for k, v in iteritems(variables))
+    return new_vars, attributes
