@@ -76,6 +76,10 @@ class _LocIndexer(object):
         self.data_array[self._remap_key(key)] = value
 
 
+# singleton object for checking
+UNNAMED = object()
+
+
 class DataArray(AbstractArray):
     """N-dimensional array with labeled coordinates and dimensions.
 
@@ -195,6 +199,19 @@ class DataArray(AbstractArray):
             obj._dataset._coord_names.discard(name)
         return obj
 
+    @classmethod
+    def _new_from_dataset_no_copy(cls, dataset, name):
+        obj = object.__new__(cls)
+        obj._dataset = dataset
+        obj._name = name
+        return obj
+
+    def _with_replaced_dataset(self, dataset):
+        obj = object.__new__(type(self))
+        obj._name = self.name
+        obj._dataset = dataset
+        return obj
+
     @property
     def dataset(self):
         """The dataset with which this DataArray is associated.
@@ -439,7 +456,7 @@ class DataArray(AbstractArray):
         array's dataset is also a variable in this array's dataset.
         """
         ds = self._dataset.copy(deep=deep)
-        return ds[self.name]
+        return self._with_replaced_dataset(ds)
 
     def __copy__(self):
         return self.copy(deep=False)
@@ -462,7 +479,7 @@ class DataArray(AbstractArray):
         DataArray.sel
         """
         ds = self._dataset.isel(**indexers)
-        return ds[self.name]
+        return self._with_replaced_dataset(ds)
 
     indexed = utils.function_alias(isel, 'indexed')
 
@@ -538,7 +555,7 @@ class DataArray(AbstractArray):
         align
         """
         ds = self._dataset.reindex(copy=copy, **indexers)
-        return ds[self.name]
+        return self._with_replaced_dataset(ds)
 
     def rename(self, new_name_or_name_dict):
         """Returns a new DataArray with renamed coordinates and/or a new name.
@@ -573,7 +590,7 @@ class DataArray(AbstractArray):
                       FutureWarning, stacklevel=2)
         names = names + (self.name,)
         ds = self._dataset.select_vars(*names)
-        return ds[self.name]
+        return self._with_replaced_dataset(ds)
 
     select = utils.function_alias(select_vars, 'select')
 
@@ -639,7 +656,7 @@ class DataArray(AbstractArray):
         """
         ds = self._dataset.copy()
         ds[self.name] = self.variable.transpose(*dims)
-        return ds[self.name]
+        return self._with_replaced_dataset(ds)
 
     def squeeze(self, dim=None):
         """Return a new DataArray object with squeezed data.
@@ -667,7 +684,29 @@ class DataArray(AbstractArray):
         numpy.squeeze
         """
         ds = self._dataset.squeeze(dim)
-        return ds[self.name]
+        return self._with_replaced_dataset(ds)
+
+    def dropna(self, dim, how='any', thresh=None):
+        """Returns a new array with dropped labels for missing values along
+        the provided dimension.
+
+        Parameters
+        ----------
+        dim : str
+            Dimension along which to drop missing values. Dropping along
+            multiple dimensions simultaneously is not yet supported.
+        how : {'any', 'all'}, optional
+            * any : if any NA values are present, drop that label
+            * all : if all values are NA, drop that label
+        thresh : int, default None
+            If supplied, require this many non-NA values.
+
+        Returns
+        -------
+        DataArray
+        """
+        ds = self._dataset.dropna(dim, how=how, thresh=thresh)
+        return self._with_replaced_dataset(ds)
 
     def reduce(self, func, dim=None, axis=None, keep_attrs=False, **kwargs):
         """Reduce this array by applying `func` along some dimension(s).
@@ -710,7 +749,7 @@ class DataArray(AbstractArray):
         ds = self._dataset.drop_vars(*drop)
         ds[self.name] = var
 
-        return ds[self.name]
+        return self._with_replaced_dataset(ds)
 
     @classmethod
     def concat(cls, *args, **kwargs):
@@ -740,10 +779,36 @@ class DataArray(AbstractArray):
         concat_over = set(concat_over) | set([name])
 
         ds = Dataset._concat(datasets, dim, indexers, concat_over=concat_over)
-        return ds[name]
+        return cls._new_from_dataset_no_copy(ds, name)
+
+    def to_pandas(self):
+        """Convert this array into a pandas object with the same shape.
+
+        The type of the returned object depends on the number of DataArray
+        dimensions:
+        * 1D -> `pandas.Series`
+        * 2D -> `pandas.DataFrame`
+        * 3D -> `pandas.Panel`
+
+        Only works for arrays with 3 or fewer dimensions.
+
+        The DataArray constructor performs the inverse transformation.
+        """
+        # TODO: consolidate the info about pandas constructors and the
+        # attributes that correspond to their indexes into a separate module?
+        constructors = {0: lambda x: x,
+                        1: pd.Series,
+                        2: pd.DataFrame,
+                        3: pd.Panel}
+        try:
+            constructor = constructors[self.ndim]
+        except KeyError:
+            raise ValueError('cannot convert arrays with %s dimensions into '
+                             'pandas objects' % self.ndim)
+        return constructor(self.values, *self.indexes.values())
 
     def to_dataframe(self):
-        """Convert this array into a pandas.DataFrame.
+        """Convert this array and its coordinates into a tidy pandas.DataFrame.
 
         The DataFrame is indexed by the Cartesian product of index coordinates
         (in the form of a :py:class:`pandas.MultiIndex`).
@@ -751,7 +816,7 @@ class DataArray(AbstractArray):
         Other coordinates are included as columns in the DataFrame.
         """
         # TODO: add a 'name' parameter
-        return self._dataset.to_dataframe()
+        return self._dataset._to_dataframe(self.dims)
 
     def to_series(self):
         """Convert this array into a pandas.Series.
@@ -764,7 +829,7 @@ class DataArray(AbstractArray):
 
     @classmethod
     def from_series(cls, series):
-        """Convert a pandas.Series into an xray.DataArray
+        """Convert a pandas.Series into an xray.DataArray.
 
         If the series's index is a MultiIndex, it will be expanded into a
         tensor product of one-dimensional coordinates (filling in missing values
@@ -774,7 +839,7 @@ class DataArray(AbstractArray):
         # TODO: add a 'name' parameter
         df = pd.DataFrame({series.name: series})
         ds = Dataset.from_dataframe(df)
-        return ds[series.name]
+        return cls._new_from_dataset_no_copy(ds, series.name)
 
     def _all_compat(self, other, compat_str):
         """Helper function for equals and identical"""
@@ -816,21 +881,36 @@ class DataArray(AbstractArray):
             return False
 
     def _result_name(self, other=None):
-        # these names are used to index a dimension, so we don't want to
-        # preserve them
-        name_is_dim = self.name in self.dims
+
+        if self.name in self.dims:
+            # these names match dimension, so if we preserve them we will also
+            # rename indexes
+            return None
+
+        if other is None:
+            # shortcut
+            return self.name
+
+        other_name = getattr(other, 'name', UNNAMED)
+        other_dims = getattr(other, 'dims', ())
+
+        if other_name in other_dims:
+            # same trouble as above
+            return None
+
         # use the same naming heuristics as pandas:
         # https://github.com/ContinuumIO/blaze/issues/458#issuecomment-51936356
-        ambiguous = (other is not None and hasattr(other, 'name')
-                     and other.name != self.name)
-        return None if name_is_dim or ambiguous else self.name
+        if other_name is UNNAMED or other_name == self.name:
+            return self.name
+
+        return None
 
     def __array_wrap__(self, obj, context=None):
         new_var = self.variable.__array_wrap__(obj, context)
         ds = self.coords.to_dataset()
         name = self._result_name()
         ds[name] = new_var
-        return ds[name]
+        return self._new_from_dataset_no_copy(ds, name)
 
     @staticmethod
     def _unary_op(f):
@@ -852,7 +932,7 @@ class DataArray(AbstractArray):
             ds[name] = (f(self.variable, other_variable)
                         if not reflexive
                         else f(other_variable, self.variable))
-            return ds[name]
+            return self._new_from_dataset_no_copy(ds, name)
         return func
 
     @staticmethod
