@@ -1,20 +1,22 @@
-from collections import Mapping
+import sys
+import gzip
+import warnings
 import functools
 from io import BytesIO
-import warnings
+from collections import Mapping
 
 import numpy as np
 import pandas as pd
 
-from .. import backends, conventions
-from . import alignment
+from . import ops
+from . import utils
 from . import common
-from . import formatting
 from . import groupby
 from . import indexing
 from . import variable
-from . import utils
-from . import ops
+from . import alignment
+from . import formatting
+from .. import backends, conventions
 from .coordinates import DatasetCoordinates, Indexes
 from .utils import (Frozen, SortedKeysDict, ChainMap,
                     multi_index_from_product)
@@ -30,7 +32,7 @@ def open_dataset(nc, decode_cf=True, mask_and_scale=True, decode_times=True,
     nc : str or file
         Path to a netCDF4 file or an OpenDAP URL (opened with python-netCDF4)
         or a file object or string serialization of a netCDF3 file (opened with
-        scipy.io.netcdf).
+        scipy.io.netcdf). If the filename ends with .gz, the file is gunzipped
     decode_cf : bool, optional
         Whether to decode these variables, assuming they were saved according
         to CF conventions.
@@ -58,19 +60,38 @@ def open_dataset(nc, decode_cf=True, mask_and_scale=True, decode_times=True,
     # move this to a classmethod Dataset.open?
     # TODO: this check has the unfortunate side-effect that
     # paths to files cannot start with 'CDF'.
-    if isinstance(nc, basestring) and not nc.startswith('CDF'):
-        # If the initialization nc is a string and it doesn't
-        # appear to be the contents of a netcdf file we load
-        # it using the netCDF4 package
-        store = backends.NetCDF4DataStore(nc, *args, **kwargs)
+    if isinstance(nc, basestring):
+        # If the initialization nc is a string and
+        if nc.endswith('.gz'):
+            # if the string ends with .gz, then gunzip and open as netcdf file
+            if sys.version_info[:2] < (2, 7):
+                raise ValueError('reading a gzipped netCDF not '
+                                 'supported on Python 2.6')
+            try:
+                store = backends.ScipyDataStore(gzip.open(nc), *args, **kwargs)
+            except TypeError as e:
+                # TODO: gzipped loading only works with NetCDF3 files.
+                if 'is not a valid NetCDF 3 file' in e.message:
+                    raise ValueError("xray: gzipped file loading only supports NetCDF 3 files.")
+                else:
+                    raise e
+        elif not nc.startswith('CDF'):
+            # nc does not appear to be a string holding the contents of a
+            # netcdf file so we treat it as a path and load it using the
+            # netCDF4 package
+            store = backends.NetCDF4DataStore(nc, *args, **kwargs)
     else:
         # If nc is a file-like object we read it using
         # the scipy.io.netcdf package
         store = backends.ScipyDataStore(nc, *args, **kwargs)
-    return Dataset.load_store(store, decode_cf=decode_cf,
-                              mask_and_scale=mask_and_scale,
-                              decode_times=decode_times,
-                              concat_characters=concat_characters)
+    if decode_cf:
+        decoder = functools.partial(conventions.cf_decoder,
+                                    mask_and_scale=mask_and_scale,
+                                    decode_times=decode_times,
+                                    concat_characters=concat_characters)
+    else:
+        decoder = None
+    return Dataset.load_store(store, decoder=decoder)
 
 
 # list of attributes of pd.DatetimeIndex that are ndarrays of time info
@@ -329,7 +350,10 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         """
         for dim, size in iteritems(self.dims):
             if dim not in self._arrays:
-                coord = variable.Coordinate(dim, np.arange(size))
+                # This is equivalent to np.arange(size), but
+                # waits to create the array until its actually accessed.
+                data = indexing.LazyIntegerRange(size)
+                coord = variable.Coordinate(dim, data)
                 self._arrays[dim] = coord
 
     def _update_vars_and_coords(self, new_arrays, new_coord_names={},
@@ -378,17 +402,14 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
                                      check_coord_names=False)
 
     @classmethod
-    def load_store(cls, store, decode_cf=True, mask_and_scale=True,
-                   decode_times=True, concat_characters=True):
+    def load_store(cls, store, decoder=None):
         """Create a new dataset from the contents of a backends.*DataStore
         object
         """
-        variables = store.variables
-        if decode_cf:
-            variables = conventions.decode_cf_variables(
-                variables, mask_and_scale=mask_and_scale,
-                decode_times=decode_times, concat_characters=concat_characters)
-        obj = cls(variables, attrs=store.attrs)
+        variables, attributes = store.load()
+        if decoder:
+            variables, attributes = decoder(variables, attributes)
+        obj = cls(variables, attrs=attributes)
         obj._file_obj = store
         return obj
 
@@ -766,10 +787,12 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
                 del obj._arrays[name]
         return obj
 
-    def dump_to_store(self, store):
+    def dump_to_store(self, store, encoder=None):
         """Store dataset contents to a backends.*DataStore object."""
-        store.set_variables(self._arrays)
-        store.set_attributes(self.attrs)
+        variables, attributes = self, self.attrs
+        if encoder:
+            variables, attributes = encoder(variables, attributes)
+        store.store(variables, attributes)
         store.sync()
 
     def to_netcdf(self, filepath, **kwdargs):
@@ -1224,6 +1247,61 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         """
         return common.squeeze(self, self.dims, dim)
 
+    def dropna(self, dim, how='any', thresh=None, subset=None):
+        """Returns a new dataset with dropped labels for missing values along
+        the provided dimension.
+
+        Parameters
+        ----------
+        dim : str
+            Dimension along which to drop missing values. Dropping along
+            multiple dimensions simultaneously is not yet supported.
+        how : {'any', 'all'}, optional
+            * any : if any NA values are present, drop that label
+            * all : if all values are NA, drop that label
+        thresh : int, default None
+            If supplied, require this many non-NA values.
+        subset : sequence, optional
+            Subset of variables to check for missing values. By default, all
+            variables in the dataset are checked.
+
+        Returns
+        -------
+        Dataset
+        """
+        # TODO: consider supporting multiple dimensions? Or not, given that
+        # there are some ugly edge cases, e.g., pandas's dropna differs
+        # depending on the order of the supplied axes.
+
+        if dim not in self.dims:
+            raise ValueError('%s must be a single dataset dimension' % dim)
+
+        if subset is None:
+            subset = list(self.vars)
+
+        count = np.zeros(self.dims[dim], dtype=int)
+        size = 0
+
+        for k in subset:
+            array = self._arrays[k]
+            if dim in array.dims:
+                dims = [d for d in array.dims if d != dim]
+                count += array.count(dims)
+                size += np.prod([self.dims[d] for d in dims])
+
+        if thresh is not None:
+            mask = count >= thresh
+        elif how == 'any':
+            mask = count == size
+        elif how == 'all':
+            mask = count > 0
+        elif how is not None:
+            raise ValueError('invalid how option: %s' % how)
+        else:
+            raise TypeError('must specify how or thresh')
+
+        return self.isel(**{dim: mask})
+
     def reduce(self, func, dim=None, keep_attrs=False, **kwargs):
         """Reduce this dataset by applying `func` along some dimension(s).
 
@@ -1266,7 +1344,7 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         variables = OrderedDict()
         for name, var in iteritems(self._arrays):
             reduce_dims = [dim for dim in var.dims if dim in dims]
-            if reduce_dims:
+            if reduce_dims or not var.dims:
                 if name not in self.coords:
                     if len(reduce_dims) == 1:
                         # unpack dimensions for the benefit of functions like
@@ -1425,6 +1503,15 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
 
         return concatenated
 
+    def _to_dataframe(self, ordered_dims):
+        columns = [k for k in self if k not in self.dims]
+        orig_arrays = (self._arrays[k] for k in columns)
+        broadcast_arrays = variable.broadcast_variables(*orig_arrays)
+        data = [arr.transpose(*ordered_dims).values.reshape(-1)
+                for arr in broadcast_arrays]
+        index = self.coords.to_index(ordered_dims)
+        return pd.DataFrame(OrderedDict(zip(columns, data)), index=index)
+
     def to_dataframe(self):
         """Convert this dataset into a pandas.DataFrame.
 
@@ -1432,22 +1519,7 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         DataFrame. The DataFrame is be indexed by the Cartesian product of
         this dataset's indices.
         """
-        columns = [k for k in self if k not in self.dims]
-        data = []
-        # we need a template to broadcast all dataset variables against
-        # using stride_tricks lets us make the ndarray for broadcasting without
-        # having to allocate memory
-        shape = tuple(self.dims.values())
-        empty_data = np.lib.stride_tricks.as_strided(np.array(0), shape=shape,
-                                                     strides=[0] * len(shape))
-        template = variable.Variable(self.dims.keys(), empty_data)
-        for k in columns:
-            _, var = variable.broadcast_variables(template, self._arrays[k])
-            _, var_data = np.broadcast_arrays(template.values, var.values)
-            data.append(var_data.reshape(-1))
-
-        index = self.coords.to_index()
-        return pd.DataFrame(OrderedDict(zip(columns, data)), index=index)
+        return self._to_dataframe(self.dims)
 
     @classmethod
     def from_dataframe(cls, dataframe):
