@@ -1,3 +1,4 @@
+from xray.conventions import cf_decoder
 try:
     import cPickle as pickle
 except ImportError:
@@ -7,6 +8,7 @@ import contextlib
 import os.path
 import tempfile
 import unittest
+import sys
 
 import numpy as np
 import pandas as pd
@@ -41,6 +43,10 @@ def create_encoded_masked_and_scaled_data():
     return Dataset({'x': ('t', [-1, -1, 0, 1, 2], attributes)})
 
 
+class CastsUnicodeToBytes(object):
+    pass
+
+
 class DatasetIOTestCases(object):
     def create_store(self):
         raise NotImplementedError
@@ -58,14 +64,18 @@ class DatasetIOTestCases(object):
                                         {'units': 'units of awesome'})
         with self.create_store() as store:
             expected.dump_to_store(store)
-            actual = Dataset.load_store(store)
+            # the test data contains times.  In case the store
+            # cf_encodes them we need to cf_decode them.
+            actual = Dataset.load_store(store, cf_decoder)
         self.assertDatasetAllClose(expected, actual)
 
     def test_write_store(self):
         expected = create_test_data()
         with self.create_store() as store:
             expected.dump_to_store(store)
-            actual = Dataset.load_store(store)
+            # the test data contains times.  In case the store
+            # cf_encodes them we need to cf_decode them.
+            actual = Dataset.load_store(store, cf_decoder)
         self.assertDatasetAllClose(expected, actual)
 
     def test_roundtrip_test_data(self):
@@ -119,21 +129,69 @@ class DatasetIOTestCases(object):
             # see the note under test_zero_dimensional_variable
             del original['nan']
         expected = original.copy(deep=True)
-        expected['letters_nans'][-1] = ''
-        if type(self) is not NetCDF4DataTest:
+        if type(self) in [NetCDF3ViaNetCDF4DataTest, ScipyDataTest]:
             # for netCDF3 tests, expect the results to come back as characters
             expected['letters_nans'] = expected['letters_nans'].astype('S')
             expected['letters'] = expected['letters'].astype('S')
         with self.roundtrip(original) as actual:
-            self.assertDatasetIdentical(expected, actual)
+            try:
+                self.assertDatasetIdentical(expected, actual)
+            except AssertionError:
+                # Most stores use '' for nans in strings, but some don't
+                # first try the ideal case (where the store returns exactly)
+                # the original Dataset), then try a more realistic case.
+                # ScipyDataTest, NetCDF3ViaNetCDF4DataTest and NetCDF4DataTest
+                # all end up using this case.
+                expected['letters_nans'][-1] = ''
+                self.assertDatasetIdentical(expected, actual)
 
     def test_roundtrip_string_data(self):
         expected = Dataset({'x': ('t', ['ab', 'cdef'])})
         with self.roundtrip(expected) as actual:
-            if type(self) is not NetCDF4DataTest:
+            if isinstance(self, CastsUnicodeToBytes):
                 expected['x'] = expected['x'].astype('S')
             self.assertDatasetIdentical(expected, actual)
 
+    def test_roundtrip_example_1_netcdf(self):
+        expected = open_example_dataset('example_1.nc')
+        with self.roundtrip(expected) as actual:
+            # we allow the attributes to differ since that
+            # will depend on the encoding used.  For example,
+            # without CF encoding 'actual' will end up with
+            # a dtype attribute.
+            self.assertDatasetEqual(expected, actual)
+
+    def test_roundtrip_example_1_netcdf_gz(self):
+        if sys.version_info[:2] < (2, 7):
+            with self.assertRaisesRegexp(ValueError,
+                                         'gzipped netCDF not supported'):
+                open_example_dataset('example_1.nc.gz')
+        else:
+            with open_example_dataset('example_1.nc.gz') as expected:
+                with open_example_dataset('example_1.nc') as actual:
+                    self.assertDatasetIdentical(expected, actual)
+
+    def test_orthogonal_indexing(self):
+        in_memory = create_test_data()
+        with self.roundtrip(in_memory) as on_disk:
+            indexers = {'dim1': np.arange(3), 'dim2': np.arange(4),
+                        'dim3': np.arange(5)}
+            expected = in_memory.isel(**indexers)
+            actual = on_disk.isel(**indexers)
+            self.assertDatasetAllClose(expected, actual)
+            # do it twice, to make sure we're switched from orthogonal -> numpy
+            # when we cached the values
+            actual = on_disk.isel(**indexers)
+            self.assertDatasetAllClose(expected, actual)
+
+    def test_pickle(self):
+        on_disk = open_example_dataset('bears.nc')
+        unpickled = pickle.loads(pickle.dumps(on_disk))
+        self.assertDatasetIdentical(on_disk, unpickled)
+
+
+class CFEncodedDataTest(DatasetIOTestCases):
+
     def test_roundtrip_strings_with_fill_value(self):
         values = np.array(['ab', 'cdef', np.nan], dtype=object)
         encoding = {'_FillValue': np.string_('X'), 'dtype': np.dtype('S1')}
@@ -144,9 +202,10 @@ class DatasetIOTestCases(object):
             self.assertDatasetIdentical(expected, actual)
 
         original = Dataset({'x': ('t', values, {}, {'_FillValue': '\x00'})})
-        if type(self) is NetCDF4DataTest:
-            # NetCDF4 should still write a VLEN (unicode) string
+        if not isinstance(self, CastsUnicodeToBytes):
+            # these stores can save unicode strings
             expected = original.copy(deep=True)
+        if type(self) is NetCDF4DataTest:
             # the netCDF4 library can't keep track of an empty _FillValue for
             # VLEN variables:
             expected['x'][-1] = ''
@@ -163,35 +222,20 @@ class DatasetIOTestCases(object):
         with self.roundtrip(decoded) as actual:
             self.assertDatasetAllClose(decoded, actual)
         with self.roundtrip(decoded, decode_cf=False) as actual:
+            # TODO: this assumes that all roundtrips will first
+            # encode.  Is that something we want to test for?
             self.assertDatasetAllClose(encoded, actual)
+        with self.roundtrip(encoded, decode_cf=False) as actual:
+            self.assertDatasetAllClose(encoded, actual)
+        # make sure roundtrip encoding didn't change the
+        # original dataset.
+        self.assertDatasetIdentical(encoded,
+                                    create_encoded_masked_and_scaled_data())
         with self.roundtrip(encoded) as actual:
             self.assertDatasetAllClose(decoded, actual)
         with self.roundtrip(encoded, decode_cf=False) as actual:
             self.assertDatasetAllClose(encoded, actual)
 
-    def test_roundtrip_example_1_netcdf(self):
-        with open_example_dataset('example_1.nc') as expected:
-            with self.roundtrip(expected) as actual:
-                self.assertDatasetIdentical(expected, actual)
-
-    def test_orthogonal_indexing(self):
-        in_memory = create_test_data()
-        with self.roundtrip(in_memory) as on_disk:
-            indexers = {'dim1': np.arange(3), 'dim2': np.arange(4),
-                        'dim3': np.arange(5)}
-            expected = in_memory.isel(**indexers)
-            actual = on_disk.isel(**indexers)
-            self.assertDatasetAllClose(expected, actual)
-            # do it twice, to make sure we're switched from orthogonal -> numpy
-            # when we cached the values
-            actual = on_disk.isel(**indexers)
-            self.assertDatasetAllClose(expected, actual)
-
-    def test_pickle(self):
-        with open_example_dataset('bears.nc') as on_disk:
-            unpickled = pickle.loads(pickle.dumps(on_disk))
-            self.assertDatasetIdentical(on_disk, unpickled)
-
 
 @contextlib.contextmanager
 def create_tmp_file(suffix='.nc'):
@@ -204,7 +248,7 @@ def create_tmp_file(suffix='.nc'):
 
 
 @requires_netCDF4
-class NetCDF4DataTest(DatasetIOTestCases, TestCase):
+class NetCDF4DataTest(CFEncodedDataTest, TestCase):
     @contextlib.contextmanager
     def create_store(self):
         with create_tmp_file() as tmp_file:
@@ -392,10 +436,57 @@ class NetCDF4DataTest(DatasetIOTestCases, TestCase):
             self.assertDatasetIdentical(data, actual)
             self.assertEqual(actual['x'].dtype, np.dtype('S4'))
 
+    def test_open_encodings(self):
+        # Create a netCDF file with explicit time units
+        # and make sure it makes it into the encodings
+        # and survives a round trip
+        with create_tmp_file() as tmp_file:
+            with nc4.Dataset(tmp_file, 'w') as ds:
+                ds.createDimension('time', size=10)
+                ds.createVariable('time', np.int32, dimensions=('time',))
+                units = 'days since 1999-01-01'
+                ds.variables['time'].setncattr('units', units)
+                ds.variables['time'][:] = np.arange(10) + 4
+
+            expected = Dataset()
+
+            time = pd.date_range('1999-01-05', periods=10)
+            encoding = {'units': units, 'dtype': np.dtype('int32')}
+            expected['time'] = ('time', time, {}, encoding)
+
+            actual = open_dataset(tmp_file)
+
+            self.assertVariableEqual(actual['time'], expected['time'])
+            actual_encoding = dict((k, v) for k, v
+                                   in iteritems(actual['time'].encoding)
+                                   if k in expected['time'].encoding)
+            self.assertDictEqual(actual_encoding, expected['time'].encoding)
+
+    def test_dump_and_open_encodings(self):
+        # Create a netCDF file with explicit time units
+        # and make sure it makes it into the encodings
+        # and survives a round trip
+        with create_tmp_file() as tmp_file:
+            with nc4.Dataset(tmp_file, 'w') as ds:
+                ds.createDimension('time', size=10)
+                ds.createVariable('time', np.int32, dimensions=('time',))
+                units = 'days since 1999-01-01'
+                ds.variables['time'].setncattr('units', units)
+                ds.variables['time'][:] = np.arange(10) + 4
+
+            xray_dataset = open_dataset(tmp_file)
+
+            with create_tmp_file() as tmp_file2:
+                xray_dataset.dump(tmp_file2)
+
+                with nc4.Dataset(tmp_file2, 'r') as ds:
+                    self.assertEqual(ds.variables['time'].getncattr('units'), units)
+                    self.assertArrayEqual(ds.variables['time'], np.arange(10) + 4)
+
 
 @requires_netCDF4
 @requires_scipy
-class ScipyDataTest(DatasetIOTestCases, TestCase):
+class ScipyDataTest(CFEncodedDataTest, CastsUnicodeToBytes, TestCase):
     @contextlib.contextmanager
     def create_store(self):
         fobj = BytesIO()
@@ -409,7 +500,7 @@ class ScipyDataTest(DatasetIOTestCases, TestCase):
 
 
 @requires_netCDF4
-class NetCDF3ViaNetCDF4DataTest(DatasetIOTestCases, TestCase):
+class NetCDF3ViaNetCDF4DataTest(CFEncodedDataTest, CastsUnicodeToBytes, TestCase):
     @contextlib.contextmanager
     def create_store(self):
         with create_tmp_file() as tmp_file:
