@@ -3,7 +3,8 @@ import pandas as pd
 from copy import deepcopy
 from textwrap import dedent
 
-from xray import concat, Dataset, DataArray, Coordinate, Variable, align
+from xray import (align, concat, broadcast_arrays, Dataset, DataArray,
+                  Coordinate, Variable)
 from xray.core.pycompat import iteritems, OrderedDict
 from . import TestCase, ReturnItem, source_ndarray, unittest
 
@@ -329,6 +330,10 @@ class TestDataArray(TestCase):
         self.assertDataArrayIdentical(da[:3], da.sel(x=slice('c')))
         self.assertDataArrayIdentical(da[:3], da.sel(x=['a', 'b', 'c']))
         self.assertDataArrayIdentical(da[:, :4], da.sel(y=(self.ds['y'] < 4)))
+        # verify that indexing with a dataarray works
+        b = DataArray('b')
+        self.assertDataArrayIdentical(da[1], da.sel(x=b))
+        self.assertDataArrayIdentical(da[[1]], da.sel(x=slice(b, b)))
 
     def test_loc(self):
         self.ds['x'] = ('x', np.array(list('abcdefghij')))
@@ -603,6 +608,14 @@ class TestDataArray(TestCase):
         expected = DataArray(1 + np.arange(3), coords=[('x', range(3))])
         self.assertDataArrayIdentical(expected, actual)
 
+        # regression tests for #254
+        actual = orig[0] < orig
+        expected = DataArray([False, True, True], coords=[('x', range(3))])
+        self.assertDataArrayIdentical(expected, actual)
+
+        actual = orig > orig[0]
+        self.assertDataArrayIdentical(expected, actual)
+
     def test_dataset_math(self):
         # more comprehensive tests with multiple dataset variables
         obs = Dataset({'tmin': ('x', np.arange(5)),
@@ -652,6 +665,25 @@ class TestDataArray(TestCase):
     def test_squeeze(self):
         self.assertVariableEqual(self.dv.variable.squeeze(), self.dv.squeeze())
 
+    def test_dropna(self):
+        x = np.random.randn(4, 4)
+        x[::2, 0] = np.nan
+        arr = DataArray(x, dims=['a', 'b'])
+
+        actual = arr.dropna('a')
+        expected = arr[1::2]
+        self.assertDataArrayIdentical(actual, expected)
+
+        actual = arr.dropna('b', how='all')
+        self.assertDataArrayIdentical(actual, arr)
+
+        actual = arr.dropna('a', thresh=1)
+        self.assertDataArrayIdentical(actual, arr)
+
+        actual = arr.dropna('b', thresh=3)
+        expected = arr[:, 1:]
+        self.assertDataArrayIdentical(actual, expected)
+
     def test_reduce(self):
         coords = {'x': [-1, -2], 'y': ['ab', 'cd', 'ef'],
                   'lat': (['x', 'y'], [[1, 2, 3], [-1, -2, -3]]),
@@ -679,6 +711,11 @@ class TestDataArray(TestCase):
         self.assertVariableEqual(self.dv.reduce(np.mean, 'x'),
                                  self.v.reduce(np.mean, 'x'))
 
+        orig = DataArray([[1, 0, np.nan], [3, 0, 3]], coords, dims=['x', 'y'])
+        actual = orig.count()
+        expected = DataArray(5, {'c': -999})
+        self.assertDataArrayIdentical(expected, actual)
+
     def test_reduce_keep_attrs(self):
         # Test dropped attrs
         vm = self.va.mean()
@@ -751,6 +788,14 @@ class TestDataArray(TestCase):
                                      grouped.reduce(np.sum, 'y'))
         self.assertDataArrayAllClose(expected_sum_axis1, grouped.sum('y'))
 
+    def test_groupby_count(self):
+        array = DataArray([0, 0, np.nan, np.nan, 0, 0],
+                          coords={'cat': ('x', ['a', 'b', 'b', 'c', 'c', 'c'])},
+                          dims='x')
+        actual = array.groupby('cat').count()
+        expected = DataArray([1, 1, 2], coords=[('cat', ['a', 'b', 'c'])])
+        self.assertDataArrayIdentical(actual, expected)
+
     @unittest.skip('needs to be fixed for shortcut=False, keep_attrs=False')
     def test_groupby_reduce_attrs(self):
         array = self.make_groupby_example_array()
@@ -848,6 +893,71 @@ class TestDataArray(TestCase):
         self.assertDataArrayIdentical(dv1, self.dv[:5])
         self.assertDataArrayIdentical(dv2, self.dv[:5])
 
+    def test_broadcast_arrays(self):
+        x = DataArray([1, 2], coords=[('a', [-1, -2])], name='x')
+        y = DataArray([1, 2], coords=[('b', [3, 4])], name='y')
+        x2, y2 = broadcast_arrays(x, y)
+        expected_coords = [('a', [-1, -2]), ('b', [3, 4])]
+        expected_x2 = DataArray([[1, 1], [2, 2]], expected_coords, name='x')
+        expected_y2 = DataArray([[1, 2], [1, 2]], expected_coords, name='y')
+        self.assertDataArrayIdentical(expected_x2, x2)
+        self.assertDataArrayIdentical(expected_y2, y2)
+
+        x = DataArray(np.random.randn(2, 3), dims=['a', 'b'])
+        y = DataArray(np.random.randn(3, 2), dims=['b', 'a'])
+        x2, y2 = broadcast_arrays(x, y)
+        expected_x2 = x
+        expected_y2 = y.T
+        self.assertDataArrayIdentical(expected_x2, x2)
+        self.assertDataArrayIdentical(expected_y2, y2)
+
+        with self.assertRaisesRegexp(ValueError, 'cannot broadcast'):
+            z = DataArray([1, 2], coords=[('a', [-10, 20])])
+            broadcast_arrays(x, z)
+
+    def test_to_pandas(self):
+        # 0d
+        actual = DataArray(42).to_pandas()
+        expected = np.array(42)
+        self.assertArrayEqual(actual, expected)
+
+        # 1d
+        values = np.random.randn(3)
+        index = pd.Index(['a', 'b', 'c'], name='x')
+        da = DataArray(values, coords=[index])
+        actual = da.to_pandas()
+        self.assertArrayEqual(actual.values, values)
+        self.assertArrayEqual(actual.index, index)
+        self.assertArrayEqual(actual.index.name, 'x')
+
+        # 2d
+        values = np.random.randn(3, 2)
+        da = DataArray(values, coords=[('x', ['a', 'b', 'c']), ('y', [0, 1])],
+                       name='foo')
+        actual = da.to_pandas()
+        self.assertArrayEqual(actual.values, values)
+        self.assertArrayEqual(actual.index, ['a', 'b', 'c'])
+        self.assertArrayEqual(actual.columns, [0, 1])
+
+        # roundtrips
+        for shape in [(3,), (3, 4), (3, 4, 5)]:
+            dims = list('abc')[:len(shape)]
+            da = DataArray(np.random.randn(*shape), dims=dims)
+            roundtripped = DataArray(da.to_pandas())
+            self.assertDataArrayIdentical(da, roundtripped)
+
+        with self.assertRaisesRegexp(ValueError, 'cannot convert'):
+            DataArray(np.random.randn(1, 2, 3, 4, 5)).to_pandas()
+
+    def test_to_dataframe(self):
+        # regression test for #260
+        arr = DataArray(np.random.randn(3, 4),
+                        [('B', [1, 2, 3]), ('A', list('cdef'))])
+        expected = arr.to_series()
+        actual = arr.to_dataframe()[None]
+        self.assertArrayEqual(expected.values, actual.values)
+        self.assertArrayEqual(expected.index.values, actual.index.values)
+
     def test_to_and_from_series(self):
         expected = self.dv.to_dataframe()['foo']
         actual = self.dv.to_series()
