@@ -8,7 +8,8 @@ except ImportError:
 import numpy as np
 import pandas as pd
 
-from xray import align, concat, backends, Dataset, DataArray, Variable
+from xray import (align, concat, conventions, backends, Dataset, DataArray,
+                  Variable)
 from xray.core import indexing, utils
 from xray.core.pycompat import iteritems, OrderedDict
 
@@ -47,22 +48,12 @@ class InaccessibleArray(utils.NDArrayMixin):
 
 
 class InaccessibleVariableDataStore(backends.InMemoryDataStore):
-    def __init__(self):
-        self.dims = OrderedDict()
-        self._variables = OrderedDict()
-        self.attrs = OrderedDict()
-
-    def set_variable(self, name, variable):
-        self._variables[name] = variable
-        return self._variables[name]
-
-    def open_store_variable(self, var):
-        data = indexing.LazilyIndexedArray(InaccessibleArray(var.values))
-        return Variable(var.dims, data, var.attrs)
-
-    @property
-    def store_variables(self):
-        return self._variables
+    def get_variables(self):
+        def lazy_inaccessible(x):
+            data = indexing.LazilyIndexedArray(InaccessibleArray(x.values))
+            return Variable(x.dims, data, x.attrs)
+        return dict((k, lazy_inaccessible(v)) for
+                    k, v in iteritems(self._variables))
 
 
 class TestDataset(TestCase):
@@ -1030,8 +1021,8 @@ class TestDataset(TestCase):
         store = InaccessibleVariableDataStore()
         create_test_data().dump_to_store(store)
 
-        for decode_cf in [False, True]:
-            ds = Dataset.load_store(store, decode_cf=decode_cf)
+        for decoder in [None, conventions.cf_decoder]:
+            ds = Dataset.load_store(store, decoder=decoder)
             with self.assertRaises(UnexpectedDataAccess):
                 ds.load_data()
             with self.assertRaises(UnexpectedDataAccess):
@@ -1041,6 +1032,66 @@ class TestDataset(TestCase):
             ds.isel(time=10)
             ds.isel(time=slice(10), dim1=[0]).isel(dim1=0, dim2=-1)
 
+    def test_dropna(self):
+        x = np.random.randn(4, 4)
+        x[::2, 0] = np.nan
+        y = np.random.randn(4)
+        y[-1] = np.nan
+        ds = Dataset({'foo': (('a', 'b'), x), 'bar': (('b', y))})
+
+        expected = ds.isel(a=slice(1, None, 2))
+        actual = ds.dropna('a')
+        self.assertDatasetIdentical(actual, expected)
+
+        expected = ds.isel(b=slice(1, 3))
+        actual = ds.dropna('b')
+        self.assertDatasetIdentical(actual, expected)
+
+        actual = ds.dropna('b', subset=['foo', 'bar'])
+        self.assertDatasetIdentical(actual, expected)
+
+        expected = ds.isel(b=slice(1, None))
+        actual = ds.dropna('b', subset=['foo'])
+        self.assertDatasetIdentical(actual, expected)
+
+        expected = ds.isel(b=slice(3))
+        actual = ds.dropna('b', subset=['bar'])
+        self.assertDatasetIdentical(actual, expected)
+
+        actual = ds.dropna('a', subset=[])
+        self.assertDatasetIdentical(actual, ds)
+
+        actual = ds.dropna('a', subset=['bar'])
+        self.assertDatasetIdentical(actual, ds)
+
+        actual = ds.dropna('a', how='all')
+        self.assertDatasetIdentical(actual, ds)
+
+        actual = ds.dropna('b', how='all', subset=['bar'])
+        expected = ds.isel(b=[0, 1, 2])
+        self.assertDatasetIdentical(actual, expected)
+
+        actual = ds.dropna('b', thresh=1, subset=['bar'])
+        self.assertDatasetIdentical(actual, expected)
+
+        actual = ds.dropna('b', thresh=2)
+        self.assertDatasetIdentical(actual, ds)
+
+        actual = ds.dropna('b', thresh=4)
+        expected = ds.isel(b=[1, 2, 3])
+        self.assertDatasetIdentical(actual, expected)
+
+        actual = ds.dropna('a', thresh=3)
+        expected = ds.isel(a=[1, 3])
+        self.assertDatasetIdentical(actual, ds)
+
+        with self.assertRaisesRegexp(ValueError, 'a single dataset dimension'):
+            ds.dropna('foo')
+        with self.assertRaisesRegexp(ValueError, 'invalid how'):
+            ds.dropna('a', how='somehow')
+        with self.assertRaisesRegexp(TypeError, 'must specify how or thresh'):
+            ds.dropna('a', how=None)
+
     def test_reduce(self):
         data = create_test_data()
 
@@ -1110,6 +1161,12 @@ class TestDataset(TestCase):
         actual = ds.argmin('x')
         self.assertDatasetIdentical(expected, actual)
 
+    def test_reduce_scalars(self):
+        ds = Dataset({'x': ('a', [2, 2]), 'y': 2, 'z': ('b', [2])})
+        expected = Dataset({'x': 0, 'y': 0, 'z': 0})
+        actual = ds.var()
+        self.assertDatasetIdentical(expected, actual)
+
     @unittest.skip('see github issue 209')
     def test_reduce_only_one_axis(self):
 
@@ -1129,6 +1186,12 @@ class TestDataset(TestCase):
         with self.assertRaisesRegexp(TypeError, 'non-integer axis'):
             ds.reduce(mean_only_one_axis, ['x', 'y'])
 
+    def test_count(self):
+        ds = Dataset({'x': ('a', [np.nan, 1]), 'y': 0, 'z': np.nan})
+        expected = Dataset({'x': 1, 'y': 1, 'z': 0})
+        actual = ds.count()
+        self.assertDatasetIdentical(expected, actual)
+
     def test_apply(self):
         data = create_test_data()
         data.attrs['foo'] = 'bar'
