@@ -90,6 +90,11 @@ class VariableSubclassTestCases(object):
         x = self.cls(['x'], pd.DatetimeIndex([d]))
         self.assertIndexedLikeNDArray(x, np.datetime64(d), 'datetime64[ns]')
 
+    def test_index_0d_not_a_time(self):
+        d = np.datetime64('NaT')
+        x = self.cls(['x'], [d])
+        self.assertIndexedLikeNDArray(x, d, None)
+
     def test_index_0d_object(self):
 
         class HashableItemWrapper(object):
@@ -587,6 +592,23 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         self.assertEqual(len(vm.attrs), len(_attrs))
         self.assertEqual(vm.attrs, _attrs)
 
+    def test_count(self):
+        expected = Variable([], 3)
+        actual = Variable(['x'], [1, 2, 3, np.nan]).count()
+        self.assertVariableIdentical(expected, actual)
+
+        v = Variable(['x'], np.array(['1', '2', '3', np.nan], dtype=object))
+        actual = v.count()
+        self.assertVariableIdentical(expected, actual)
+
+        actual = Variable(['x'], [True, False, True]).count()
+        self.assertVariableIdentical(expected, actual)
+        self.assertEqual(actual.dtype, 'int64')
+
+        expected = Variable(['x'], [2, 3])
+        actual = Variable(['x', 'y'], [[1, 0, np.nan], [1, 1, 1]]).count('y')
+        self.assertVariableIdentical(expected, actual)
+
 
 class TestCoordinate(TestCase, VariableSubclassTestCases):
     cls = staticmethod(Coordinate)
